require "net/http"
require "rexml/document"
require 'optparse'
require "feed.rb"
require "yaml"

class Jisha
  private_class_method :new
  attr_accessor :feeds, :parser, :feeds_list
  @@jisha = nil

  def self.usage
    puts "Usage: jisha command"
    puts "\t where command can be one of the following:"    
    puts "\t\t search  <image_name>\t Search the image cache"
    puts "\t\t install <image_name>\t Downloads and starts VM image"
    puts "\t\t remove  <image_name>\t Stops VM image and deletes its associated storage\n"
    puts "\t\t start   <image_name>\t Starts an image that has been installed is currently stopped"
    puts "\t\t stop    <image_name>\t Stops an image that is currently running"
    puts "\t\t update\t\t\t Updates the image cache with images advertised over feeds in image_feeds.list"
    puts "\t\t clean\t\t\t Wipe the image cache"
  end
  
  def self.create
    @@jisha = new unless @@jisha    
    @@jisha
  end

  def initialize
      ParserFactory.load( "feed_parsers" )
      @parser = ParserFactory.parser_for( "RSS" )
      @feeds = YAML::load( File.open( '.cache', 'r') )
  end
  
  def update( feeds_list )
    @feeds = Array.new
    
    File.open( feeds_list, 'r') do |file|
      file.each_line do |line|
        feed = Feed.new( line, @parser )
        feed.update 
        @feeds.push( feed )
      end
    end
    
    File.open( '.cache', 'w') do |file|
      file << @feeds.to_yaml 
    end
    
  end
  
  def search( query )
    unless @feeds
      puts "Please build your image cache by running ./jisha.rb update"
      Process.exit
    end
    
    puts "Results: \n\n"
    hits = 0
    @feeds.each do |feed|
      images = feed.images.find_all do |image| 
        search_regex = Regexp.new( query )
        image.title =~ search_regex or image.distro =~ search_regex or image.description =~ search_regex
      end
      puts "#{images}"
    end
    
  end
  
  def install( image_name )
    unless @feeds
      puts "Please build your image cache by running ./jisha.rb update"
      Process.exit
    end
    
    @feeds.each do |feed|
      image = feed.images.find { |image| image.title == image_name }
      if image
        puts "found image: #{image.title}"
        image.download 
      else
        puts "Could not find #{image_name} in the image cache"
      end
    end
  end
  
  def remove
  end
  
  def clean
    @feeds = nil
    File.truncate( '.cache', 0 )
    puts "Jisha's image cache has been wiped clean"
  end
  
  def images
    images = Array.new
    
    @feeds.each do |feed|
      images << feed.images
      feed.images.each { |image| puts "#{image.to_s}"}
    end
    
    return images    
  end
end

if ARGV.length < 1 or ARGV.length > 2
  Jisha.usage
  Process.exit
end

jisha = Jisha.create

case ARGV[0]
  when "update"
    jisha.update 'image_feeds.list'
  when "clean"
    jisha.clean
  when "install"
    unless ARGV[1] 
      Jisha.usage
      Process.exit
    end
    jisha.install ARGV[1]
  when "search"
    unless ARGV[1] 
      Jisha.usage
      Process.exit
    end
    jisha.search ARGV[1]    
  when "cache_dump"
    puts jisha.feeds
end
